SCHEMA_APIS = 'apis'
SCHEMA_PATH = 'path'

FILE_EXT_JSON = 'json'
FILE_EXT_YAML = 'yaml'

VALID_FILE_EXT = [
    '.'+FILE_EXT_JSON,
    '.'+FILE_EXT_YAML,
]
SWAGGER_FILE_NAMES = [
    'resource_list' + '.' + FILE_EXT_JSON,
    'swagger' + '.' + FILE_EXT_JSON,
    'swagger' + '.' + FILE_EXT_YAML,
]

SCOPE_SEPARATOR = '!##!'
